<?php

/*
    daily_container.inc - Part of the daily module for Drupal
    Copyright (c) 2004-2006  Frodo Looijaard <drupal@frodo.looijaard.name>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_container.inc
 * Functions specific for the node type daily-container
 *
 * All function names in this module are prepended with _daily_container
 */


/**
 * hook_access: Define access restrictions for node type daily-container
 *
 *   - view: Access is denied if this node is not yet published.
 *   - create: Access depends on the 'create daily items' permission
 *   - update: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 *   - delete: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 * @note See daily_perm() for the defined user permissions by this module.
 * @note See hook_access() for a description of parameters and return values.
 */
function daily_container_access($op, $node) {
  global $user;
  if ($node->uid == 1) {
    return true;
  }
  switch ($op) {
    case 'create':
      return user_access('create daily items');
    case 'update':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
    case 'delete':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
  }
}

/**
 * hook_insert: Respond to node insertion for nodetype daily-container.
 *
 * Updates the database table {daily_container}.
 * @note See hook_insert() for a description of parameters and return values.
 */
function daily_container_insert($node) {
  db_query("INSERT INTO {daily_container} (vid, nid, tid, nr) VALUES (%d, %d, %d, %d)", $node->vid, $node->nid, $node->tid, $node->nr);
}

/**
 * hook_update: Respond to node updating for nodetype daily-container
 *
 * Updates the database table {daily_container}
 * @note See hook_update() for a description of parameters and return values.
 */
function daily_container_update($node) {
  if ($node->old_vid) {
    db_query("INSERT INTO {daily_container} (vid, nid, tid, nr) VALUES (%d, %d, %d, %d)", $node->vid, $node->nid, $node->tid, $node->nr);
  } 
  else {
    db_query("UPDATE {daily} SET date='%s', autopublish=%d WHERE vid=%d", _daily_pack_date($node->pubdate), $node->autopublish, $node->vid);
  }
}


/**
 * hook_delete: Respond to node deletion for nodetype daily-container.
 *
 * Updates the database table {daily_container}
 * @note See hook_delete() for a description of parameters and return values.
 */
function daily_container_delete(&$node) {
  // We use the nid field here because we need to remove all revisions
  // and there is no way to get the revision id's.
  // Hopefully, this will change in the future.
  db_query("DELETE FROM {daily_container} WHERE nid = %d", $node->nid);
}

/**
 * hook_load: Load node-type-specific information for nodetype daily-container.
 *
 * This is ugly. Load the contained daily node, unless we are editing (in which
 * case we load the real daily container node).
 * Note that the normal API is that we return any additional fields. In the
 * case of containers, we want to return the contained object in most cases,
 * and discard the original object completely. To do that, we actually
 * modify our $node parameter.
 *
 * If editing, or if no node can be displayed because there is none in the
 * selected category, the following field are added:
 *   - tid: The taxonomy term id of the daily collection for this container
 *   - nr: The offset within the collection (1 = first, -1 = last)
 *
 * In all other cases, the node object is modified to contain the daily node
 * to be displayed. The following fields are added:
 *   - container: The container node, which is the original node to which
 *                the same fields as above are added.
 *   - title_org: The original title (not used, but still).
 *
 * @note See hook_load() for a description of parameters and return values.
 */
function daily_container_load(&$node) {
  $data = db_fetch_object(db_query("SELECT tid, nr FROM {daily_container} WHERE vid = %d", $node->vid));

  // Ugly hack. If we are editing a node, we need the container itself.
  if (arg(2) == "edit" or arg(2) == "delete" or arg(2) == "action" or arg(2) == "revisions") {
   return $data;
  }

  $daily_node = _daily_node_by_tid($data->tid, $data->nr);

  // If there is not a single daily for this tid, return the container itself
  if (! isset($daily_node)) {
    return $data;
  }

  // Modify the $node parameter to contain the node we just got.
  $node->tid = $data->tid;
  $node->nr = $data->nr;
  $daily_node->container = $node;
  $daily_node->title_org = $daily_node->title;
  $daily_node->title = $node->title;
  $node = $daily_node;
}

/** hook_prepare: Prepare a node for editing
 *
 * This is always called: when starting a new node, when editing an existing
 * node, when previewing a node and when submitting a node
 * @note See hook_prepare() for a description of parameters and return values.
 */
function daily_container_prepare(&$node) {
  if (!module_exist("taxonomy")) {
    form_set_error('fatal',  _daily_form_error_no_taxonomy());
    return;
  }

  $vid = variable_get("daily_vocabulary", 0);   
  if (!$vid) {     
    form_set_error('fatal', _daily_form_error_no_vocabulary());
    return;
  }
}

/**
 * hook_validate: Verify a node editing form for nodetype daily-container.
 *
 * Note that some things can only be verified after a POST (preview/submit).
 * All input is validated strictly.
 * @note See hook_validate() for a description of parameters and return values.
 */
function daily_container_validate(&$node) {
  # If we have already form errors flagged, we will not do any further
  # checking
  if (form_get_errors()) {
    return;
  }

  $vid = variable_get("daily_vocabulary", 0);   
  $term = module_invoke('taxonomy', 'get_term', $node->tid);
  if (!$node->tid or !$term or ($term->vid != $vid)) {
    form_set_error('tid', _daily_form_error_no_tid());
  }
  if ($node->nr !== (string) (int) $node->nr or $node->nr != floor($node->nr)) {
    form_set_error('nr', _daily_form_error_correct_nr());
  }
}

/**
 * hook_form: Display a node editing form for nodetype daily-container.
 *
 * @note See hook_form() for a description of parameters and return values.
 */
function daily_container_form(&$node, &$param) {

  $vid = variable_get("daily_vocabulary", 0);
  $query = db_query("SELECT td.tid, td.name FROM {term_data} td WHERE td.vid = %d ORDER BY td.name", $vid);

  while ($db_record = db_fetch_object($query)) {
    $options[$db_record->tid] = $db_record->name;
  }

  # PHP5 complains if no options are present
  if (!count($options)) {
    $options[0] = '<none>';
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $node->title,
    '#size' => 60,
    '#maxlength' => 128,
    '#description' => NULL,
    '#attributes' => NULL,
    '#required' => TRUE,
  );

  $form['tid'] = array(
    '#type' => 'select',
    '#title' => t("Vocabulary term"),
    '#default_value' => $node->tid,
    '#options' => $options,
    '#description' => t("The daily item class"),
    '#extra' => 0,
    '#multiple' => FALSE,
    '#required' => TRUE,
  );

  $form["nr"] = array(
    '#type' => 'textfield',
    '#title' => t("Position"),
    '#default_value' => $node->nr,
    '#size' => 3,
    '#maxlength' => 5,
    '#description' => t("The position within the class. Use -1 for the last item, -2 for the last-but-one item, etc; use 1 for the first item, 2 for the second item, etc; use 0 for a random item refreshed once a day."),
    '#attributes' => NULL,
    '#required' => TRUE,
  );

  if (variable_get("daily_use_log_message", 0)) {
    $form['log'] = array(
      '#type' => 'textarea',
      '#title' => t('Log message'),
      '#default_value' => $node->log,
      '#cols' => 60,
      '#rows' => 5,
      '#description' => t('An explanation of the additions or updates being made to help other authors understand your motivations.'),
    );
  }


  return $form;
}

/**
 * hook_view: View a node for nodetype daily-container.
 *
 * Note that this will only be called when we are previewing!
 *
 * @note See hook_view() for a description of parameters and return values.
 */
function daily_container_view(&$node, $teaser = FALSE, $page = FALSE) {
  # When indexing, we do not want to display anything here
  if (!strstr($_SERVER['PHP_SELF'], 'cron.php')) {
    if ($node->nr) {
      $node->body = "<i>The ". abs($node->nr) . ($node->nr % 10 == 1?"st":($node->nr % 10 == 2?"nd":($node->nr % 10 == 3?"rd":"th"))) ." node will be displayed here, counted from the ". ($node->nr < 0?"most recent":"oldest") .".</i>";
    } 
    else {
      $node->body = "<i>A random node will be displayed here</i>";
    }
  }
}

# vim:sw=2:ft=php:

?>
