<?php

/*
    daily_daily.inc - Part of the daily module for Drupal
    Copyright (c) 2004-2006  Frodo Looijaard <drupal@frodo.looijaard.name>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/** @file daily_daily.inc
 * Functions specific for the node type daily-daily
 *
 * All function names in this module are prepended with _daily_daily
 */


/**
 * hook_access: Define access restrictions for node type daily-daily
 *
 *   - view: Access is denied if this node is not yet published.
 *           Access is denied if this node's date is in the future and
 *           you do not have the 'view future daily items' permission.
 *   - create: Access depends on the 'create daily items' permission
 *   - update: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 *   - delete: Access is allowed if you authored this node, and you have
 *             the 'edit own daily items' permission.
 * @note See daily_perm() for the defined user permissions by this module.
 * @note See hook_access() for a description of parameters and return values.
 */
function daily_daily_access($op, $node) {
  global $user;
  switch ($op) {
    case 'create':
      return user_access('create daily items');
    case 'update':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
    case 'delete':
      return user_access('edit own daily items') && ($user->uid == $node->uid);
  }
}

/**
 * hook_insert: Respond to node insertion for nodetype daily-daily.
 *
 * Updates the database table {daily}.
 * @note See hook_insert() for a description of parameters and return values.
 */
function daily_daily_insert($node) {
  db_query("INSERT INTO {daily} (vid, nid, date, autopublish) VALUES (%d, %d, '%s', %d)", $node->vid, $node->nid, _daily_pack_date($node->pubdate), $node->autopublish);
}

/**
 * hook_update: Respond to node updating for nodetype daily-daily.
 *
 * Updates the database table {daily}
 * @note See hook_update() for a description of parameters and return values.
 */
function daily_daily_update($node) {
  if ($node->old_vid) {
    db_query("INSERT INTO daily (vid,nid,date,autopublish) VALUES (%d,%d,'%s',%d)", $node->vid, $node->nid, _daily_pack_date($node->pubdate), $node->autopublish);

    $query = db_query("SELECT n.title,d.nid FROM {daily} d,{node} n,term_node t WHERE t.tid = %d AND t.nid = d.nid AND n.vid = d.vid AND n.nid != %d AND d.date = '%s'", $node->tid, $node->nid, _daily_pack_date($node->pubdate));
    if (db_num_rows($query)) {
      $message = 'WARNING: by reverting to this node revision, you have introduced duplicate publication dates. Duplicate nodes may be inaccessible through the displayed calendar. You are encouraged to change the date on either <a href="node/'. $node->nid .'/edit">the current node</a> or on one of these:';
      while ($db_record = db_fetch_object($query)) {
	$message .= '<br/ >&nbsp;&nbsp;&nbsp;&nbsp;<a href="node/'.$db_record->nid.'/edit">'. $db_record->title. '</a>';
      }
      drupal_set_message($message);
    }
  } 
  else {
    db_query("UPDATE {daily} SET date='%s', autopublish=%d WHERE vid=%d", _daily_pack_date($node->pubdate), $node->autopublish, $node->vid);
  }
}

/**
 * hook_delete: Respond to node deletion for nodetype daily-daily.
 *
 * Updates the database table {daily}
 * @note See hook_delete() for a description of parameters and return values.
 */
function daily_daily_delete(&$node) {
  // We use the nid field here because we need to remove all revisions
  // and there is no way to get the revision id's.
  // Hopefully, this will change in the future.
  db_query("DELETE FROM {daily} WHERE nid = %d", $node->nid);
}

/**
 * hook_load: Load node-type-specific information for nodetype daily-daily.
 *
 * Adds the following fields to the node object:
 *   - year: The year of the node date (int)
 *   - month: The month of the node date (int, 1..12)
 *   - day: The day of the node date (int, 1..31)
 *   - tid: The taxonomy term id within the vocabulary associated with daily.
 *   - term_name: The taxonomy term name of tid.
 * @note See hook_load() for a description of parameters and return values.
 */
function daily_daily_load($node) {
  $vocid = variable_get("daily_vocabulary", 0);
  $db_record = db_fetch_object(db_query("SELECT d.date, d.autopublish, t.tid, td.name FROM {daily} d,{term_node} t,{term_data} td WHERE d.vid = %d AND t.nid = %d AND td.tid = t.tid AND td.vid = %d", $node->vid, $node->nid, $vocid));
  $data->pubdate = _daily_unpack_date($db_record->date);
  $data->tid = $db_record->tid;
  $data->term_name = $db_record->name;
  $data->autopublish = $db_record->autopublish;
  return $data;
}


/** hook_prepare: Prepare a node for editing
 *
 * This is always called: when starting a new node, when editing an existing
 * node, when previewing a node and when submitting a node
 * @note See hook_prepare() for a description of parameters and return values.
 */
function daily_daily_prepare(&$node) {
  if (!module_exist("taxonomy")) {
    form_set_error('fatal',  _daily_form_error_no_taxonomy());
    return;
  }
  
  $vid = variable_get("daily_vocabulary", 0);
  if (!$vid) {
    form_set_error('fatal', _daily_form_error_no_vocabulary());
    return;
  }

  if (!isset($node->pubdate)) {
    $node->pubdate = _daily_today();
  }
}

/**
 * hook_validate: Verify a node editing form for nodetype daily-daily.
 *
 * Note that some things can only be verified after a POST (preview/submit).
 * All input is validated strictly.
 * @note See hook_validate() for a description of parameters and return values.
 */
function daily_daily_validate($node) {
  # If we have already form errors flagged, we will not do any further
  # checking
  if (form_get_errors()) {
    return;
  }

  # Check whether the date is valid
  if (! array_key_exists("year", $node->pubdate) or ! array_key_exists("month", $node->pubdate) or ! array_key_exists("day", $node->pubdate)) {
    form_set_error('pubdate', "Invalid form submission");
  } 
  else {
    $date = $node->pubdate;
    if ($date["year"] < 1970 or $date["year"] > 2050) {
      form_set_error('pubdate', "Invalid year selected");
    }
    elseif ($date["month"] < 1 or $date["month"] > 12) {
        form_set_error('pubdate', "Invalid month selected (form submission problem?)");
    }
    elseif ($date["day"] < 1 or $date["day"] > _daily_days_per_month($date["month"], $date["year"])) {
      form_set_error('pubdate', "Invalid day of month selected");
    }
  }

  /* Not so nice. We have to find our tid. But all we have is the taxonomy
   * field in our node. It is an array of tids. Presumably, one belongs to
   * us. But which one?
   * To make things worse, if the daily taxonomy has not been defined
   * properly, we may not have gotten a tid at all. But we have to
   * differentiate that from the case where there has been no POST yet.
   * So we use a heuristic here. */
  if (array_key_exists('edit', $_POST)) {
    $tids = _daily_find_our_tids($node->taxonomy);
    if (count($tids) == 0) {
      form_set_error('warning', _daily_form_error_no_tid());
    }
    elseif (count($tids) > 1) {
      form_set_error('warning', _daily_form_error_several_tids());
    }
    /* Check for a duplicate. It is a duplicate if it has the same date and
     * a different node id, provided it has the same taxonomy term id */
    elseif (db_result(db_query("SELECT * from {daily} d,{node} n, {term_node} t WHERE d.vid = n.vid AND n.nid = t.nid AND d.date = '%s' AND t.tid = %d AND n.nid <> %d", _daily_pack_date($node->pubdate), $tids[0], $node->nid))) {
        form_set_error('pubdate', _daily_form_error_duplicate_date());
    }
  }
}

function daily_daily_submit(&$node) {
  if ($node->autopublish) {
    $node->status = _daily_pack_date($node->pubdate) <= _daily_pack_date(_daily_today())?1:0;
  }

  $tids = _daily_find_our_tids($node->taxonomy);
  $node->tid = $tids[0];
  $data = db_fetch_object(db_query("SELECT name FROM {term_data} WHERE tid = %d", $tids[0]));
  $node->term_name = $data->name;
}


/**
 * hook_form: Display a node editing form for nodetype daily-daily.
 *
 * @note See hook_form() for a description of parameters and return values.
 */
function daily_daily_form(&$node, &$param) {
  $form = array();

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $node->title,
    '#size' => 60,
    '#maxlength' => 128,
    '#required' => TRUE,
  );

  $form['pubdate'] = array(
    '#type' => 'date',
    '#title' => t("Date"),
    '#default_value' => $node->pubdate,
    '#description' => t("The publication date"),
    '#required' => TRUE,
  );

  $form['autopublish'] = array(
    '#type' => 'checkbox',
    '#title' => t("Publish automatically"),
    '#return_value' => 1,
    '#default_value' => (isset($node->autopublish)?$node->autopublish:variable_get("daily_default_autopublish", 1)),
    '#description' => t("Publish this note automatically on the selected date"),
  );

  $form['body'] = array(
    '#type' => 'textarea',
    '#title' => t("Body"),
    '#default_value' => $node->body,
    '#cols' => 60,
    '#rows' => 20,
    '#description' => '',
    '#attributes' => NULL,
    '#required' => TRUE,
  );

  if (variable_get("daily_use_log_message", 0)) {
    $form['log'] = array(
      '#type' => 'textarea',
      '#title' => t('Log message'),
      '#default_value' => $node->log,
      '#cols' => 60,
      '#rows' => 5,
      '#description' => t('An explanation of the additions or updates being made to help other authors understand your motivations.'),
    );
  }

  $form['format'] = filter_form($node->format); 

  return $form;
}

/**
 * hook_view: View a node for nodetype daily-daily.
 *
 * @note See hook_view() for a description of parameters and return values.
 */
function daily_daily_view(&$node, $teaser = FALSE, $page = FALSE) {
  /* When creating the search index, we do not want to add extra data */
  if (!strstr($_SERVER['PHP_SELF'], 'cron.php')) {
    /* If we are previewing, we may lack the term_name and tid fields */
    if (! isset($node->term_name)) {
      $tids = _daily_find_our_tids($node->taxonomy);
      $node->tid = $tids[0];
      $data = db_fetch_object(db_query("SELECT name FROM {term_data} WHERE tid = %d", $tids[0]));
      $node->term_name = $data->name;
    }
    _daily_content($node, $teaser, $page);
  }
}

# vim:sw=2:ft=php:

?>
